@@ -1,4 +1,4 @@
-# Copyright 2004-2015 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2017 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -31,6 +31,7 @@ from renpy.display.render import blit_lock, IDENTITY, BLIT, DISSOLVE, IMAGEDISSO
 # A map from cached surface to rle version of cached surface.
 rle_cache = weakref.WeakKeyDictionary()
 
+
 class Clipper(object):
     """
     This is used to calculate the clipping rectangle and update rectangles
@@ -171,7 +172,6 @@ class Clipper(object):
         while sized:
             area, x0, y0, x1, y1 = sized.pop()
 
-
             merged = False
 
             if nca + area >= sa:
@@ -234,6 +234,7 @@ class Clipper(object):
 
 clippers = [ Clipper() ]
 
+
 def surface(w, h, alpha):
     """
     Creates a surface that shares a pixel format with the screen. The created
@@ -247,13 +248,15 @@ def surface(w, h, alpha):
 
     return rv.subsurface((2, 2, w, h))
 
+
 def copy_surface(surf):
     w, h = surf.get_size()
     rv = surface(w, h, True)
 
-    renpy.display.accelerator.nogil_copy(surf, rv) # @UndefinedVariable
+    renpy.display.accelerator.nogil_copy(surf, rv)  # @UndefinedVariable
     return rv
 
+
 def draw_special(what, dest, x, y):
     """
     This handles the special drawing operations, such as dissolve and
@@ -428,7 +431,6 @@ def draw(dest, clip, what, xo, yo, screen):
             # what.draw_func(newdest, newx, newy)
             draw_special(what, newdest, newx, newy)
 
-
         return
 
     # Deal with clipping, if necessary.
@@ -490,10 +492,11 @@ def draw(dest, clip, what, xo, yo, screen):
     for child, cxo, cyo, _focus, _main in what.visible_children:
         draw(dest, clip, child, xo + cxo, yo + cyo, screen)
 
+
 def draw_transformed(dest, clip, what, xo, yo, alpha, forward, reverse):
 
     # If our alpha has hit 0, don't do anything.
-    if alpha <= 0.003: # (1 / 256)
+    if alpha <= 0.003:  # (1 / 256)
         return
 
     if forward is None:
@@ -638,7 +641,6 @@ def draw_transformed(dest, clip, what, xo, yo, alpha, forward, reverse):
         draw_transformed(dest, clip, child, xo + cxo, yo + cyo, alpha * what.alpha * what.over, child_forward, child_reverse)
 
 
-
 def do_draw_screen(screen_render, full_redraw, swdraw):
     """
     Draws the render produced by render_screen to the screen.
@@ -690,7 +692,6 @@ class SWDraw(object):
         self.mouse_backing_pos = None
         self.mouse_info = None
 
-
         # Is the mouse currently visible?
         self.mouse_old_visible = None
 
@@ -719,6 +720,9 @@ class SWDraw(object):
         # 1px border around it iff we're scaling.
         self.window = None
 
+        # Did we show fullscreen video in the last frame?
+        self.showing_video = False
+
     def set_mode(self, virtual_size, physical_size, fullscreen):
 
         # Reset before resize.
@@ -855,7 +859,7 @@ class SWDraw(object):
         if self.mouse_location:
             updates.append(self.hide_mouse())
 
-        if tex and pos and renpy.game.interface.mouse_focused: # @UndefinedVariable
+        if tex and pos and renpy.game.interface.mouse_focused:  # @UndefinedVariable
             updates.append(self.show_mouse(pos, info))
 
         return updates
@@ -946,47 +950,57 @@ class SWDraw(object):
 
         return True
 
-
     def draw_screen(self, surftree, fullscreen_video):
         """
         Draws the screen.
         """
 
-        if not fullscreen_video:
+        if fullscreen_video:
 
-            updates = [ ]
+            if not self.showing_video:
+                self.window.fill((0, 0, 0, 255))
 
-            updates.extend(self.draw_mouse(False))
+            w, h = self.window.get_size()
+            frame = renpy.display.video.render_movie("movie", w, h)
 
-            damage = do_draw_screen(surftree, self.full_redraw, self)
+            if frame is not None:
+                surftree = frame
 
-            if damage:
-                updates.extend(damage)
-
-            self.full_redraw = False
-
-            if self.window is self.screen:
-
-                updates.extend(self.draw_mouse(True))
-                pygame.display.update(updates)
-
-            else:
-
-                if self.scale_fast:
-                    pygame.transform.scale(self.window, self.screen.get_size(), self.screen)
-                else:
-                    renpy.display.scale.smoothscale(self.window, self.screen.get_size(), self.screen)
-
-                self.draw_mouse(True)
-                pygame.display.flip()
+            self.full_redraw = True
+            self.showing_video = True
 
         else:
+            self.showing_video = False
+
+        updates = [ ]
+
+        updates.extend(self.draw_mouse(False))
+
+        damage = do_draw_screen(surftree, self.full_redraw, self)
+
+        if damage:
+            updates.extend(damage)
+
+        self.full_redraw = False
+
+        if self.window is self.screen:
+
+            updates.extend(self.draw_mouse(True))
+            pygame.display.update(updates)
+
+        else:
+
+            if self.scale_fast:
+                pygame.transform.scale(self.window, self.screen.get_size(), self.screen)
+            else:
+                renpy.display.scale.smoothscale(self.window, self.screen.get_size(), self.screen)
+
+            self.draw_mouse(True)
             pygame.display.flip()
+
+        if fullscreen_video:
             self.full_redraw = True
 
-        self.suppressed_blit = fullscreen_video
-
-
     def render_to_texture(self, render, alpha):
 
         rv = surface(render.width, render.height, alpha)
@@ -1017,7 +1031,6 @@ class SWDraw(object):
             if what.forward:
                 cx, cy = what.forward.transform(cx, cy)
 
-
             if isinstance(child, renpy.display.render.Render):
                 if self.is_pixel_opaque(child, x, y):
                     return True
@@ -1035,7 +1048,6 @@ class SWDraw(object):
 
         return False
 
-
     def mutated_surface(self, surf):
         """
         Called to indicate that the given surface has changed.
@@ -1081,7 +1093,6 @@ class SWDraw(object):
         self.mutated_surface(surf)
         return surf
 
-
     def free_memory(self):
         """
         Frees up memory.
@@ -1098,7 +1109,7 @@ class SWDraw(object):
 
         return
 
-    def quit(self): #@ReservedAssignment
+    def quit(self):  # @ReservedAssignment
         """
         Shuts down the drawing system.
         """
